from typing import Optional, cast

from birdeyepy.utils import BirdEyeApiUrls, BirdEyeRequestParams, IHttp


class Token:
    def __init__(self, http: IHttp) -> None:
        self.http = http

    def list_all(
        self,
        *,
        sort_by: Optional[str] = "v24hUSD",
        sort_type: Optional[str] = "desc",
        offset: Optional[int] = 0,
        limit: Optional[int] = 50,
        min_liquidity: Optional[int] = 50,
    ) -> list:
        """Get token list of any supported chains.

        :param sort_by:         The field to sort by.
        :param sort_type:       The type of sorting.
        :param offset:          The offset
        :param limit:           The limit
        :param min_liquidity:   The minimum liquidity to check.
        """
        params = {
            "sort_by": sort_by,
            "sort_type": sort_type,
            "offset": offset,
            "limit": limit,
            "min_liquidity": min_liquidity,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_TOKEN_LIST, **request)

        return cast(list, response)



from birdeyepy.resources import RESOURCE_MAP
from birdeyepy.utils import BASE_BIRD_EYE_API_URL, RequestsClient


__version__ = "0.0.1"


class BirdEye:
    """API Client for BirdEye"""

    def __init__(self, api_key: str) -> None:
        _http = RequestsClient(
            base_url=BASE_BIRD_EYE_API_URL,
            headers={"X-API-KEY": api_key, "User-Agent": f"birdeyepy/v{__version__}"},
        )

        for resource_name, resource_class in RESOURCE_MAP.items():
            setattr(self, resource_name, resource_class(http=_http))



import abc
from typing import Any


class IHttp(abc.ABC):
    """Interface for http requests libs"""

    @abc.abstractmethod
    def send(self, path: str, method: str = "get", **extra: Any) -> dict | list:
        raise NotImplementedError



from typing import Optional, cast

from birdeyepy.utils import (
    DEFAULT_SOL_ADDRESS,
    BirdEyeApiUrls,
    BirdEyeRequestParams,
    IHttp,
    as_api_args,
)


class DeFi:
    def __init__(self, http: IHttp) -> None:
        self.http = http

    @as_api_args
    def price(
        self,
        *,
        address: Optional[str] = DEFAULT_SOL_ADDRESS,
        check_liquidity: Optional[int] = 100,
        include_liquidity: Optional[bool] = None,
    ) -> list:
        """Get price update of a token.

        :param address:             The address of the token.
        :param check_liquidity:     The minimum liquidity to check.
        :param include_liquidity:   Include liquidity in the response.
        """
        params = {"address": address, "check_liquidity": check_liquidity}

        if include_liquidity is not None:
            params["include_liquidity"] = include_liquidity

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_PRICE, **request)

        return cast(list, response)

    def history(
        self,
        *,
        time_from: int,
        time_to: int,
        address: Optional[str] = DEFAULT_SOL_ADDRESS,
        address_type: Optional[str] = "token",
        type_in_time: Optional[str] = "15m",
    ) -> dict:
        """Get historical price line chart of a token.

        :param time_from:       Specify the start time using Unix timestamps in seconds
        :param time_to:         Specify the end time using Unix timestamps in seconds
        :param address:         The address of the token.
        :param address_type:    The type of the address...defaults to 'token'
        :param type_in_time:    The type of time...defaults to '15m'
        """
        params = {
            "address": address,
            "address_type": address_type,
            "type": type_in_time,
            "time_from": time_from,
            "time_to": time_to,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_HISTORY_PRICE, **request)

        return cast(dict, response)



import functools
from typing import Any, Callable


def as_api_args(func: Callable) -> Callable:
    """decorator to convert arguments to format expected by API

    boolean values are converted to lowercase strings
    list values are converted to comma-separated strings
    """

    @functools.wraps(func)
    def wrapper_as_api_args(*args: int, **kwargs: Any) -> Any:
        # check in **kwargs for lists and booleans
        for kwarg in kwargs:
            value = kwargs[kwarg]

            # check if arg is list and convert it to comma-separated string
            if isinstance(value, list):
                value = ",".join(value)
            # check if arg is boolean and convert it to string
            elif isinstance(value, bool):
                value = str(value).lower()

            kwargs[kwarg] = value

        return func(*args, **kwargs)

    return wrapper_as_api_args
